@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2019, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -26,6 +26,8 @@
 #import "MFMForms.h"
 #include "mforms/textentry.h"
 
+//----------------------------------------------------------------------------------------------------------------------
+
 /**
  * A special formatter to implement text length limits.
  */
@@ -36,6 +38,8 @@
 
 @end
 
+//----------------------------------------------------------------------------------------------------------------------
+
 @implementation LimitedTextFieldFormatter
 
 - (instancetype) init
@@ -45,27 +49,37 @@
   return self;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 - (void) setMaximumLength: (int) len
 {
   maxLength = len;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 - (int) maximumLength
 {
   return maxLength;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 - (NSString *) stringForObjectValue: (id) object
 {
   return (NSString *) object;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 - (BOOL) getObjectValue: (id *) object forString: (NSString *) string errorDescription: (NSString **) error
 {
   *object = string;
   return YES;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 - (BOOL) isPartialStringValid: (NSString *) partialString newEditingString: (NSString **) newString
              errorDescription: (NSString **) error
 {
@@ -78,6 +92,8 @@
   return YES;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 - (NSAttributedString *) attributedStringForObjectValue: (id) anObject
                                   withDefaultAttributes: (NSDictionary *) attributes
 {
@@ -86,7 +102,7 @@
 
 @end
 
-//--------------------------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------------------------------------------
 
 // In order to avoid duplicate code (as all these functions must be in all text entry controls)
 // this part is defined using a macro. However this is not well readable nor debuggable, so we might
@@ -130,9 +146,8 @@
     { @selector(cancelOperation:), mforms::EntryEscape }, \
     { 0 } \
   }; \
- \
-  for (int i = 0; events[i].selector != 0; i++) \
-  { \
+  \
+  for (int i = 0; events[i].selector != 0; i++) {\
     if (aSelector == events[i].selector) \
     { \
       mOwner->action(events[i].action); \
@@ -142,12 +157,16 @@
   return NO; \
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 @interface SecureTextField : NSSecureTextField  <NSTextFieldDelegate> {
   mforms::TextEntry *mOwner;
 }
 
 @end
 
+//----------------------------------------------------------------------------------------------------------------------
+
 @implementation SecureTextField
 
 - (instancetype)initWithObject: (mforms::TextEntry*)owner type: (mforms::TextEntryType)type
@@ -163,12 +182,39 @@
   return self;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 STANDARD_FOCUS_HANDLING(self) // Notify backend when getting first responder status.
 STANDARD_TEXT_ENTRY_HANDLING
 
+//----------------------------------------------------------------------------------------------------------------------
+
+- (NSAccessibilityRole)accessibilityRole {
+  return NSAccessibilityTextFieldRole;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+- (NSAccessibilityRole)accessibilitySubrole {
+  return NSAccessibilitySecureTextFieldSubrole;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+- (NSString *)accessibilityValue {
+  std::string value = mOwner->get_string_value();
+  return [NSString stringWithUTF8String: value.c_str()];
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+- (void)setAccessibilityValue: (NSString*)value {
+  mOwner->set_value(value.UTF8String);
+}
+
 @end
 
-//--------------------------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------------------------------------------
 
 @interface SearchTextField : NSSearchField  <NSSearchFieldDelegate> {
   mforms::TextEntry *mOwner;
@@ -177,6 +223,8 @@ STANDARD_TEXT_ENTRY_HANDLING
 
 @end
 
+//----------------------------------------------------------------------------------------------------------------------
+
 @implementation SearchTextField
 
 - (instancetype)initWithObject: (mforms::TextEntry*)owner type: (mforms::TextEntryType)type
@@ -199,12 +247,33 @@ STANDARD_TEXT_ENTRY_HANDLING
   return self;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 STANDARD_FOCUS_HANDLING(self) // Notify backend when getting first responder status.
 STANDARD_TEXT_ENTRY_HANDLING
 
+//----------------------------------------------------------------------------------------------------------------------
+
+- (NSAccessibilityRole)accessibilityRole {
+  return NSAccessibilityTextFieldRole;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+- (NSString *)accessibilityValue {
+    std::string value = mOwner->get_string_value();
+    return [NSString stringWithUTF8String: value.c_str()];
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+- (void)setAccessibilityValue: (NSString*)value {
+    mOwner->set_value(value.UTF8String);
+}
+
 @end
 
-//--------------------------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------------------------------------------
 
 @interface StandardTextField : NSTextField  <NSTextFieldDelegate> {
   mforms::TextEntry *mOwner;
@@ -213,6 +282,8 @@ STANDARD_TEXT_ENTRY_HANDLING
 
 @end
 
+//----------------------------------------------------------------------------------------------------------------------
+
 @implementation StandardTextField
 
 - (instancetype)initWithObject: (mforms::TextEntry*)owner type: (mforms::TextEntryType)type
@@ -232,9 +303,28 @@ STANDARD_TEXT_ENTRY_HANDLING
 STANDARD_FOCUS_HANDLING(self) // Notify backend when getting first responder status.
 STANDARD_TEXT_ENTRY_HANDLING
 
+//----------------------------------------------------------------------------------------------------------------------
+
+- (NSAccessibilityRole)accessibilityRole {
+  return NSAccessibilityTextFieldRole;
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+- (NSString *)accessibilityValue {
+    std::string value = mOwner->get_string_value();
+    return [NSString stringWithUTF8String: value.c_str()];
+}
+
+//----------------------------------------------------------------------------------------------------------------------
+
+- (void)setAccessibilityValue: (NSString*)value {
+    mOwner->set_value(value.UTF8String);
+}
+
 @end
 
-//--------------------------------------------------------------------------------------------------
+//----------------------------------------------------------------------------------------------------------------------
 
 @interface MFTextEntryImpl ()
 {
@@ -242,10 +332,13 @@ STANDARD_TEXT_ENTRY_HANDLING
 
 @end
 
+//----------------------------------------------------------------------------------------------------------------------
+
 @implementation MFTextEntryImpl
 
 @end
 
+//----------------------------------------------------------------------------------------------------------------------
 
 static bool entry_create(mforms::TextEntry *self, mforms::TextEntryType type)
 {
@@ -276,24 +369,31 @@ static bool entry_create(mforms::TextEntry *self, mforms::TextEntryType type)
   return true;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static void entry_set_text(mforms::TextEntry *self, const std::string &text)
 {
   NSTextField* entry = self->get_data();
   entry.stringValue = wrap_nsstring(text);
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static void entry_set_placeholder_text(mforms::TextEntry *self, const std::string &text)
 {
   NSTextField* entry = self->get_data();
   [entry.cell setPlaceholderString: wrap_nsstring(text)];
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static void entry_set_max_length(mforms::TextEntry *self, int maxlen)
 {
   NSTextField* entry = self->get_data();
   [[entry.cell formatter] setMaximumLength: maxlen];
 }
 
+//----------------------------------------------------------------------------------------------------------------------
 
 static std::string entry_get_text(mforms::TextEntry *self)
 {
@@ -305,22 +405,29 @@ static std::string entry_get_text(mforms::TextEntry *self)
   return "";
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static void entry_set_read_only(mforms::TextEntry *self, bool flag)
 {
   NSTextField* entry = self->get_data();
   entry.editable = flag ? NO : YES;
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static void entry_set_placeholder_color(mforms::TextEntry *self, const std::string &color)
 {
   // Not needed. The search cell automatically selects a good text color.
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static void entry_set_bordered(mforms::TextEntry *self, bool flag)
 {
   // Ignored on OS X.
 }
 
+//----------------------------------------------------------------------------------------------------------------------
 
 static void entry_cut(mforms::TextEntry *self)
 {
@@ -329,6 +436,8 @@ static void entry_cut(mforms::TextEntry *self)
   [editor cut: nil];
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static void entry_copy(mforms::TextEntry *self)
 {
   NSTextField* entry = self->get_data();
@@ -336,6 +445,8 @@ static void entry_copy(mforms::TextEntry *self)
   [editor copy: nil];
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static void entry_paste(mforms::TextEntry *self)
 {
   NSTextField* entry = self->get_data();
@@ -343,6 +454,8 @@ static void entry_paste(mforms::TextEntry *self)
   [editor paste: nil];
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static void entry_select(mforms::TextEntry *self, const base::Range &range)
 {
   NSTextField* entry = self->get_data();
@@ -350,6 +463,8 @@ static void entry_select(mforms::TextEntry *self, const base::Range &range)
   editor.selectedRange = NSMakeRange(range.position, range.size);
 }
 
+//----------------------------------------------------------------------------------------------------------------------
+
 static base::Range entry_get_selection(mforms::TextEntry *self)
 {
   NSTextField* entry = self->get_data();
@@ -358,6 +473,7 @@ static base::Range entry_get_selection(mforms::TextEntry *self)
   return base::Range(r.location, r.length);
 }
 
+//----------------------------------------------------------------------------------------------------------------------
 
 void cf_textentry_init()
 {
@@ -378,5 +494,4 @@ void cf_textentry_init()
   f->_textentry_impl.get_selection = &entry_get_selection;
 }
 
-
-
+//----------------------------------------------------------------------------------------------------------------------
