@@ -1,19 +1,23 @@
-# Copyright (c) 2012, 2015, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2012, 2018, Oracle and/or its affiliates. All rights reserved.
 #
-# This program is free software; you can redistribute it and/or
-# modify it under the terms of the GNU General Public License as
-# published by the Free Software Foundation; version 2 of the
-# License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License, version 2.0,
+# as published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-# GNU General Public License for more details.
+# This program is also distributed with certain software (including
+# but not limited to OpenSSL) that is licensed under separate terms, as
+# designated in a particular file or component or in included license
+# documentation.  The authors of MySQL hereby grant you an additional
+# permission to link the program and your derivative works with the
+# separately licensed software that they have included with MySQL.
+# This program is distributed in the hope that it will be useful,  but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+# the GNU General Public License, version 2.0, for more details.
 #
 # You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
-# 02110-1301  USA
+# along with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 
 from __future__ import with_statement
 import platform
@@ -31,13 +35,16 @@ import mforms
 
 import paramiko
 from workbench.log import log_warning, log_error, log_debug, log_debug2, log_debug3, log_info
-from wb_common import SSHFingerprintNewError
+from wb_common import SSHFingerprintNewError, format_bad_host_exception
+
+import grt
 
 SSH_PORT = 22
 REMOTE_PORT = 3306
 
 # timeout for closing an unused tunnel
-TUNNEL_TIMEOUT = 60
+TUNNEL_TIMEOUT = 3
+SSH_CONNECTION_TIMEOUT = 10
 
 # paramiko 1.6 didn't have this class
 if hasattr(paramiko, "WarningPolicy"):
@@ -100,7 +107,6 @@ class Tunnel(threading.Thread):
         with self.lock:
             return self.connecting
 
-
     def run(self):
         try:
             self.do_run()
@@ -239,6 +245,9 @@ class Tunnel(threading.Thread):
 
     def _get_ssh_config_path(self):
         paths = []
+        user_path = grt.root.wb.options.options['pathtosshconfig'] if grt.root.wb.options.options['pathtosshconfig'] is not None else None
+        if user_path:
+            paths.append(user_path)
         if platform.system().lower() == "windows":
             paths.append("%s\ssh\config" % mforms.App.get().get_user_data_folder())
             paths.append("%s\ssh\ssh_config" % mforms.App.get().get_user_data_folder())
@@ -252,6 +261,7 @@ class Tunnel(threading.Thread):
         else:
             log_debug3("ssh config file not found")
             return None
+
     def _connect_ssh(self):
         """Create the SSH client and set up the connection.
         
@@ -294,12 +304,9 @@ class Tunnel(threading.Thread):
             has_key = bool(self._keyfile)
             self._client.connect(self._server[0], self._server[1], username=self._username,
                                  key_filename=self._keyfile, password=self._password,
-                                 look_for_keys=has_key, allow_agent=has_key)
+                                 look_for_keys=has_key, allow_agent=has_key, timeout=SSH_CONNECTION_TIMEOUT)
         except paramiko.BadHostKeyException, exc:
-            if platform.system().lower() == "windows":
-                self.notify_exception_error('ERROR', "%s\nDelete entries for the host from the %s file" % (str(exc), '%s\ssh\known_hosts' % mforms.App.get().get_user_data_folder()))
-            else:
-                self.notify_exception_error('ERROR', "%s\nDelete entries for the host from the ~/.ssh/known_hosts file" % str(exc))
+            self.notify_exception_error('ERROR',format_bad_host_exception(exc, '%s\ssh\known_hosts' % mforms.App.get().get_user_data_folder() if platform.system().lower() == "windows" else "~/.ssh/known_hosts file"))
             return False
         except paramiko.BadAuthenticationType, exc:
             self.notify_exception_error('ERROR', "Bad authentication type, the server is not accepting this type of authentication.\nAllowed ones are:\n %s" % exc.allowed_types, sys.exc_info());
@@ -314,7 +321,7 @@ class Tunnel(threading.Thread):
             self.notify_exception_error('ERROR', "Error connecting SSH channel.\nPlease refer to logs for details: %s" % str(exc), sys.exc_info())
             return False
         except SSHFingerprintNewError, exc:
-            self.notify_exception_error('KEY_ERROR', { 'msg': "The authenticity of host '%(0)s (%(0)s)' can't be established.\nECDSA key fingerprint is %(1)s\nAre you sure you want to continue connecting?"  % {'0': "%s:%s" % (self._server[0], self._server[1]), '1': exc.fingerprint}, 'obj': exc})
+            self.notify_exception_error('KEY_ERROR', { 'msg': "The authenticity of host '%(0)s (%(0)s)' can't be established.\n%(1)s key fingerprint is %(2)s\nAre you sure you want to continue connecting?"  % {'0': "%s:%s" % (self._server[0], self._server[1]), '1': exc.key.get_name(), '2': exc.fingerprint}, 'obj': exc})
             return False
         except IOError, exc:
             #Io should be report to the user, so maybe he will be able to fix this issue
@@ -359,7 +366,7 @@ class Tunnel(threading.Thread):
             local_sock.close()
             return
 
-        self.notify('INFO', 'Tunnel now open %r -> %r -> %r' % (local_sock.getpeername(), sshchan.getpeername(), self._target))
+        self.notify('INFO', 'Tunnel now open %r -> %r -> %r' % (local_sock.getsockname(), sshchan.getpeername(), self._target))
 
         self._connections.append((local_sock, sshchan))
 
@@ -420,7 +427,7 @@ class TunnelManager:
 
         if found:
             with tunnel.lock:
-                print 'Reusing tunnel at port %d' % tunnel.local_port
+                log_debug('Reusing tunnel at port %d' % tunnel.local_port)
                 return tunnel.local_port
         else:
             tunnel = Tunnel(Queue.Queue(), server, username, target, password, keyfile)
@@ -437,6 +444,7 @@ class TunnelManager:
         if not tunnel:
             return 'Could not find a tunnel for port %d' % port
         error = None
+        close_tunnel = False
         tunnel.port_is_set.wait()
         if tunnel.isAlive():
             while True:
@@ -466,11 +474,13 @@ class TunnelManager:
                             error = "Server key has been stored"
                         else:
                             error = "User cancelled"
+                        close_tunnel = True
                         break # Exit returning the error message
                     elif msg_type == 'IO_ERROR':
                         error = msg
                         break # Exit returning the error message
-
+                    else:
+                        time.sleep(0.3)
                     _msg = msg
                     if type(msg) is tuple:
                         msg = '\n' + ''.join(traceback.format_exception(*msg))
@@ -479,11 +489,15 @@ class TunnelManager:
                     if msg_type == 'ERROR':
                         error = _msg
                         break  # Exit returning the error message
-
-                if not tunnel.is_connecting() or not tunnel.isAlive():
+                
+                if (not tunnel.is_connecting() or not tunnel.isAlive()) and tunnel.q.empty():  
                     break
                 time.sleep(0.3)
         log_debug("returning from wait_connection(%s): %s\n" % (port, error))
+        # we need to close tunnel so it get opened again, without it we may have problems later
+        if close_tunnel:
+            tunnel.close()
+            del self.tunnel_by_port[port]
         return error
 
 
@@ -529,10 +543,11 @@ class TunnelManager:
         else:
             self.outpipe.write(code + '\n')
         self.outpipe.flush()
-
+    
     def shutdown(self):
         for tunnel in self.tunnel_by_port.itervalues():
             tunnel.close()
+            tunnel.join()
 
     # FIXME: It seems that this function is never called. Should we remove it?
     def wait_requests(self):
