@@ -1,17 +1,41 @@
+/*
+ * Copyright (c) 2009, 2018, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License, version 2.0,
+ * as published by the Free Software Foundation.
+ *
+ * This program is also distributed with certain software (including
+ * but not limited to OpenSSL) that is licensed under separate terms, as
+ * designated in a particular file or component or in included license
+ * documentation.  The authors of MySQL hereby grant you an additional
+ * permission to link the program and your derivative works with the
+ * separately licensed software that they have included with MySQL.
+ * This program is distributed in the hope that it will be useful,  but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+ * the GNU General Public License, version 2.0, for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation, Inc.,
+ * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+ */
+
 #pragma once
 
 #include "grtui/wizard_progress_page.h"
+#include "db_plugin_be.h"
 
-class FetchSchemaContentsProgressPage : public WizardProgressPage {
+class FetchSchemaContentsProgressPage : public grtui::WizardProgressPage {
 public:
-  FetchSchemaContentsProgressPage(WizardForm *form, const char *name = "fetchSchema")
-    : WizardProgressPage(form, name, true) {
+  FetchSchemaContentsProgressPage(grtui::WizardForm *form, const char *name = "fetchSchema")
+  : grtui::WizardProgressPage(form, name, true) {
     set_title(_("Retrieve and Reverse Engineer Schema Objects"));
     set_short_title(_("Retrieve Objects"));
 
-    add_async_task(_("Retrieve Objects from Selected Schemata"),
+    add_async_task(_("Retrieve Objects from Selected Schemas"),
                    std::bind(&FetchSchemaContentsProgressPage::perform_fetch, this),
-                   _("Retrieving object lists from selected schemata..."));
+                   _("Retrieving object lists from selected schemas..."));
 
     add_task(_("Check Results"), std::bind(&FetchSchemaContentsProgressPage::perform_check, this),
              _("Checking Retrieved data..."));
